#!/usr/bin/php

<?php

/**
 * Classes handling the EPP communication with IT-NIC.
 *
 * PHP version 5
 *
 * LICENSE:
 *
 * Copyright (c) 2009, Günther Mair <guenther.mair@hoslo.ch>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3) Neither the name of Günther Mair nor the names of its contributors may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * @category    Net
 * @package     Net_EPP_IT_Client
 * @author      Günther Mair <guenther.mair@hoslo.ch>
 * @license     http://opensource.org/licenses/bsd-license.php New BSD License
 *
 */
 
 /***
 
 Modified by Fabio Ferrero <ferrero@interim.it> on 2012-07-04 (new EPP version)
 Modified by Fabio Ferrero <ferrero@interim.it> on 2010-09-22
 
 One single file stripped down version to only check credit and message queue, warnings are sent by mail, useful for crontab
 
 Depend on: the PEAR package HTTP_Client (handles communication and cookies); install it using "pear install HTTP_Client"
 
 ***/

if ( ! class_exists('HTTP_Client') ) require_once 'HTTP/Client.php';
  
function main() {
  global $basedir;
  $nic= new Net_EPP_IT_Client("./config.xml");
  $basedir = $nic->EPPCfg->basedir;
  $session= new Net_EPP_IT_Session($nic);

  if ( ! $session->hello() ) {
    echo "Connection FAILED.\n";
    print_r( $session->result );
  } else {
    if ( $session->login() === FALSE ) {
      $msg = "Login FAILED (code ".$session->svCode.", '".$session->svMsg."').\n";
      mmail($nic->EPPCfg->mailto, 'EPP NIC warning', $msg, $nic->EPPCfg->mailto);
    } else {
      $msg= '';
      $msgq= $session->pollMessageCount();
      $msxml= $session->getMessageXML();
      $credit= sprintf("%.2f", $session->showCredit());
      if ($msgq > 0) {
        if ($msgq == 1) {
          $msg .= sprintf("C'e' un messaggio in coda\n\n");
        } else {
          $msg .= sprintf("Ci sono %s messaggi in coda\n\n", $msgq, $msxml->response->msgQ->attributes()->id);
        }
        for ($i = 1; $i <= $msgq ; $i++) {
		  $msxml= $session->getMessageXML();
		  $namespaces = $msxml->getDocNamespaces(TRUE);
		  foreach($namespaces as $a => $b) {
			$msxml->registerXPathNamespace($a, $b);
		  }
		  if ($result = @$msxml->xpath('//extdom:name')) {
			while(list( , $node) = each($result)) {
				$epp_DOMAIN = $node;
			}
		  } else  {
			$result = @$msxml->xpath('//domain:name');
			while(list( , $node) = each($result)) {
				$epp_DOMAIN = $node;
			}
		  }
		  switch ($msxml->response->msgQ->msg) {
			case "autoRenewPeriod is expired":
				$uret = $session->updateExpire(utf8_decode($epp_DOMAIN));
				if ($uret == 1000) {
					$msg .= sprintf("%s: %s (expiration updated)\n", $epp_DOMAIN, $msxml->response->msgQ->msg);
					$ackret = $session->poolack($msxml,1);
				} else {
					$msg .= sprintf("%s: %s !ERROR in expiration update!\n%s\n\n", $epp_DOMAIN, $msxml->response->msgQ->msg, $uret);
					$ackret = $session->poolack($msxml);
				}
				break;
			case "DNS check ended successfully":
			case "DNS check ended successfully with warning":
				$msg .= sprintf("%s: %s\n", $epp_DOMAIN, $msxml->response->msgQ->msg);
				$uret = $session->insertDomain($epp_DOMAIN);
				if ($uret == 1000) {
					$msg .= sprintf("\tDomain %s was inserted into the database\n", $epp_DOMAIN);
				} elseif ($uret == "NOTHINGTODO") {
					$msg .= sprintf("\tDomain %s already exists in the database\n", $epp_DOMAIN);
				} else {
					$msg .= sprintf("%s: %s !an ERROR has occurred!\n%s\n\n", $epp_DOMAIN, $msxml->response->msgQ->msg, $uret);
				}
				$ackret = $session->poolack($msxml);
				break;
			case "Domain transfer has been executed":
			case "Domain transfer is expired: transfer has been executed":
				list($uret, $trstep) = $session->transferDomain($epp_DOMAIN);
				if ($uret == 1001) {
					$msg .= sprintf("%s: %s (transfer executed and domain is updated)\n%s\n\n", $epp_DOMAIN, $msxml->response->msgQ->msg, $trstep);
					$ackret = $session->poolack($msxml,1);
				} else {
					$msg .= sprintf("%s: %s !ERROR in domain update!\n%s\n%s\n\n", $epp_DOMAIN, $msxml->response->msgQ->msg, $uret, $trstep);
					$ackret = $session->poolack($msxml);
				}
				break;
			case "autoRenewPeriod is started":
			case "dnsHold is started":
				$msg .= sprintf("%s: %s\n", $epp_DOMAIN, $msxml->response->msgQ->msg);
				$ackret = $session->poolack($msxml);
				break;
			default:
				$msg .= $epp_DOMAIN.": Messaggio non gestito: ".$msxml->response->msgQ->attributes()->id." - ".$msxml->response->msgQ->msg."\n";
				$ackret = $session->poolack($msxml);
		  }
		  if ($ackret != 1000) {
			  $msg .= sprintf("ID: %s\tMSG: %s\t ACKRET: %s\n%s", $msxml->response->msgQ->attributes()->id, $msxml->response->msgQ->msg, $ackret, $msxml->asXML());
		  }

		  $session->poll();
        }
      }
      if ($credit < $nic->EPPCfg->creditwarning) {
		$creditf = 'credit.txt';
		if (file_exists($creditf)) {
			$oldcredit = file_get_contents($creditf);
		} else {
			$oldcredit = $credit;
		}
		if ($credit < $oldcredit) {
				$msg .= sprintf("ATTENZIONE! Il tuo credito e' di %s euro", $credit);
		}
		file_put_contents($creditf,$credit);
      }
      if (strlen($msg)) {
        mmail($nic->EPPCfg->mailto, 'EPP NIC warning', $msg, $nic->EPPCfg->mailto);
      }
      if ( ! $session->logout() ) {
        $msg =  "Logout FAILED (code ".$session->svCode.", '".$session->svMsg."').\n";
        mmail($nic->EPPCfg->mailto, 'EPP NIC warning', $msg, $nic->EPPCfg->mailto);
      }
    }
  }
}

function mmail($to, $subj, $msg, $from) {
  $hdr= sprintf("From: %s\nReturn-Path: %s\nReply-To: %s\n", $from, $from, $from);
  $env= sprintf("-r%s", $from);
  mail($to, $subj, $msg, $hdr, $env);
}

/*** Abstract object ***/

abstract class Net_EPP_IT_AbstractObject {
  protected $client;
  protected $trues = array("true", "TRUE", 1);
  protected $falses = array("false", "FALSE", 0, null);

  public    $xmlQuery;  // xml query string
  public    $result;    // HTTP response string
  public    $xmlResult; // parsed reponse (SimpleXMLElement may be incomplete)

  public    $svCode;
  public    $svMsg;
  public    $extValueReasonCode;
  public    $extValueReason;

  public function __construct(&$client) {
    $this->client= $client;
  }

  protected function error($msg, $code= "0000") {
    $this->svMsg= $msg;
    $this->svCode= $code;
  }

  protected function ExecuteQuery($clTRType, $clTRObject) {
    // send request + parse response
    $this->result= $this->client->sendRequest($this->xmlQuery);
    $this->xmlResult= $this->client->parseResponse($this->result['body']);

    // look for a server response code
    if ( is_object($this->xmlResult->response->result) ) {

      // look for a server message
      if ( is_object($this->xmlResult->response->result->msg) )
        $this->svMsg= $this->xmlResult->response->result->msg;
      else
        $this->svMsg= "";

      // look for a server message code
      $this->svCode= $this->xmlResult->response->result['code'];
      switch ( substr($this->svCode, 0, 1) ) {
        case "0":
        case "1":
          $return_code= TRUE;
          break;
        case "2":
        default:
          $return_code= FALSE;
          break;
      }

      // look for an extended server error message and code
      if ( is_object($this->xmlResult->response->result->extValue->reason) ) {
        $this->extValueReasonCode= $this->xmlResult->response->result->extValue->value->reasonCode;
        $this->extValueReason= $this->xmlResult->response->result->extValue->reason;
      } else {
        $this->extValueReasonCode= '';
        $this->extValueReason= '';
      }

    } else {
      $this->error("Unexpected result (no xml response code).");
      $return_code = FALSE;
    }

    return $return_code;
  }

}

/*** EPP IT Client ***/

class Net_EPP_IT_Client {
  var $EPPCfg;
  var $EPPClient;

  private $DEFAULT_HEADERS= array('content-type' => 'text/xml; charset=UTF-8');

  function __construct($cfg= "config.xml") {
    if ( is_readable($cfg) ) {
      $this->EPPCfg= @simplexml_load_file($cfg);
    } else {
      $this->EPPCfg= @simplexml_load_string($cfg);
      if ( $this->EPPCfg === FALSE ) {
        exit("FATAL ERROR: config file '".$cfg."' not readable or not a XML string\n");
      }
    }
    $this->EPPClient= new HTTP_Client(null, $this->DEFAULT_HEADERS);
  }

  function sendRequest($data) {
    if ( $this->EPPClient->valid() ) $this->EPPClient->next();
    $this->EPPClient->post($this->EPPCfg->server, utf8_encode($data), true);
    return $this->fetchResponse();
  }

  function fetchResponse() {
    return $this->EPPClient->current();
  }

  function parseResponse($xml= null) {
    if ( $xml == null ) {
      $response = $this->fetchResponse();
      return @simplexml_load_string($response[body], 'SimpleXMLElement', LIBXML_NOXMLDECL);
    } else {
      return @simplexml_load_string($xml, 'SimpleXMLElement', LIBXML_NOXMLDECL);
    }
  }

  function __destruct() {
    if ( is_object($this->EPPClient) ) $this->EPPClient->reset();
  }

}

/*** EPP IT Session ***/

class Net_EPP_IT_Session extends Net_EPP_IT_AbstractObject {
  protected $credit= null;
  protected $messages= null;
  public $dblink = null;
  public $dberror = null;

  public function hello() {
  global $basedir;
    $this->xmlQuery= file_get_contents($basedir.'hello.xml');
    $this->ExecuteQuery("hello", "");
    if ( (substr($this->result['code'], 0, 1) == "2") && (is_object($this->xmlResult->greeting)) ) {
      return TRUE;
    } else {
      return FALSE;
    }
  }

  private function loginout($which) {
    if ( $this->ExecuteQuery($which, "") ) {
      if ( is_object($this->xmlResult->response->extension) ) {
        $tmp= $this->xmlResult->response->extension->children('http://www.nic.it/ITNIC-EPP/extepp-2.0');
        $this->credit= (float) $tmp->creditMsgData->credit;
      }
      return TRUE;
    } else {
      return FALSE;
    }
  }

  public function dbconn() {
    /* Connecting, selecting database */
    $this->dblink = mysql_connect($this->client->EPPCfg->dbhost,$this->client->EPPCfg->dbuser,$this->client->EPPCfg->dbpass, true);
	if (!$this->dblink) { 
		$this->dberror = "Error in db connection: ".mysql_error();
		return FALSE;
	}
    if (!mysql_select_db($this->client->EPPCfg->dbname)) { 
		$this->dberror = "Error in db selection: ".mysql_error();
		return FALSE;
	}
	return TRUE;
  }

  public function dbclose() {
	mysql_close($this->dblink);
	return TRUE;
  }

  public function login() {
  global $basedir;
    $xml= file_get_contents($basedir.'login.xml');
    $xml= str_replace('$$REG-NAME$$', $this->client->EPPCfg->username, $xml);
    $xml= str_replace('$$REG-PWD$$', $this->client->EPPCfg->password, $xml);
    $this->xmlQuery= $xml;
    return $this->loginout('login');
  }

  public function keepalive() {
    return $this->hello();
  }

  public function logout() {
  global $basedir;
    $xml= file_get_contents($basedir.'logout.xml');
    $this->xmlQuery= $xml;
    return $this->loginout('logout');
  }

  public function pollMessageCount() {
    $this->poll();
    return (int)$this->messages;
  }

  public function getMessageXML() {
    return $this->xmlResult;
  }

  public function poll() {
  global $basedir;
    $this->xmlQuery= file_get_contents($basedir."poll.xml");
    $qrs= $this->ExecuteQuery("poll", "poll");
    if ( is_object($this->xmlResult->response->msgQ[0]) ) {
      $this->messages= (int)$this->xmlResult->response->msgQ->attributes()->count;
    } else if ( $qrs === TRUE ) {
      $this->messages= 0;
    }
    return $qrs;
  }

  public function poolack($msgxml,$actionedflag=0) {
  global $basedir;
    $xml= file_get_contents($basedir.'poolack.xml');
    $xml= str_replace('$$MSGID$$', $msgxml->response->msgQ->attributes()->id, $xml);
	$epp_DATETIME = date('Y-m-d H:i:s', strtotime($msgxml->response->msgQ->qDate));
	$epp_MSGXML = "";
	$lines = explode("\n", $msgxml->asXML(), 2);
	if(!preg_match('/^\<\?xml/', $lines[0]))
		$epp_MSGXML .= $lines[0];
	$epp_MSGXML .= $lines[1];
//	$epp_MSGXML = $msgxml->asXML()
    $this->xmlQuery= $xml;
	if ( $qrs= $this->ExecuteQuery("poll", "ack") ) {
		if ($this->dbconn()) {;
			$query = 'INSERT INTO messages (MSGID, DATETIME, MESSAGE, MSGXML, READFLAG, ACKFLAG, ACTIONEDFLAG) values("'.$msgxml->response->msgQ->attributes()->id.'", "'.$epp_DATETIME.'", "'.$msgxml->response->msgQ->msg.'", "'.addslashes($epp_MSGXML).'",1,1,'.$actionedflag.')';
			if (!mysql_query($query,$this->dblink)) {
				$this->dberror = "Query '".$query."' error: ".mysql_error();
				$this->dbclose();
				return $this->dberror;
			}
			$this->dbclose();
			return $this->svCode;
		} else {
			return $this->dberror;
		}
	} else {
		return $this->error;
	}
  }

  public function updateExpire($domainname) {
  global $basedir;
    $xml= file_get_contents($basedir.'infodomain.xml');
    $xml= str_replace('$$DOMAIN$$', $domainname, $xml);
	$this->xmlQuery= $xml;
	if ( $qrs= $this->ExecuteQuery("domain", "info") ) {
		if ($this->svCode == 1000) {
			$msgxml = $this->getMessageXML();
			$namespaces = $msgxml->getDocNamespaces(TRUE);
			foreach($namespaces as $a => $b) {
				$msgxml->registerXPathNamespace($a, $b);
			}
			$result = @$msgxml->xpath('//domain:exDate');
			while(list( , $node) = each($result)) {
				$epp_EXPIRE = date('Y-m-d H:i:s', strtotime($node));
			}
			if ($this->dbconn()) {;
				$query = 'UPDATE domains SET EXPIRE = "'.$epp_EXPIRE.'" WHERE DOMAINNAME = "'.$domainname.'"';
				if (!mysql_query($query,$this->dblink)) {
					$this->dberror = "Query '".$query."' error: ".mysql_error();
					$this->dbclose();
					return $this->dberror;
				}
				$this->dbclose();
				return $this->svCode;
			} else {
				return $this->dberror;
			}
		} else {
			return $this->svCode;
		}
	} else {
		return $qrs;
	}
  }

  public function insertDomain($domainname) {
  global $basedir;
	if ($this->dbconn()) {;
		$query = "SELECT DOMAINNAME FROM domains WHERE DOMAINNAME = '".$domainname."'";
		if (!$result = mysql_query($query,$this->dblink)) {
			$this->dberror = "Query '".$query."' error: ".mysql_error();
			$this->dbclose();
			return $this->dberror;
		}
		$row = mysql_fetch_array($result); 
		$num_results = mysql_num_rows($result); 
		if ($num_results == 1){
			$this->dbclose();
			return "NOTHINGTODO";
		}
	} else {
		return $this->dberror;
	}
    $xml= file_get_contents($basedir.'infodomain.xml');
    $xml= str_replace('$$DOMAIN$$', $domainname, $xml);
	$this->xmlQuery= $xml;
	if ( $qrs= $this->ExecuteQuery("domain", "info") ) {
		if ($this->svCode == 1000) {
			$msgxml = $this->getMessageXML();
			$namespaces = $msgxml->getDocNamespaces(TRUE);
			foreach($namespaces as $a => $b) {
				$msgxml->registerXPathNamespace($a, $b);
			}
			$epp_REGISTRANT = rtrim($this->findREM($msgxml,'registrant','//domain:infData/domain:registrant','node'));
			$epp_ADMIN = join(";",split("\n",rtrim($this->findREM($msgxml,'admin','//domain:infData/domain:contact[@type="admin"]','node'))));
			$epp_TECH = join(";",split("\n",rtrim($this->findREM($msgxml,'tech','//domain:infData/domain:contact[@type="tech"]','node'))));
			$epp_NAMESERVER = join(";",split("\n",rtrim($this->findREM($msgxml,'ns','//domain:infData/domain:ns/domain:hostAttr/domain:hostName','node'))));
			$epp_AUTHINFO = rtrim($this->findREM($msgxml,'authInfo','//domain:infData/domain:authInfo/domain:pw','node'));
			$epp_EXIPIRE = date('Y-m-d H:i:s', strtotime($this->findREM($msgxml,'exDate','//domain:infData/domain:exDate','node')));
			if ($this->dbconn()) {;
				$query = "INSERT INTO domains (`DOMAINNAME`, `REGISTRANT`, `ADMIN`, `TECH`, `NAMESERVER`, `AUTHINFO`, `STATUS`, `EXPIRE`) VALUES 
					('".$domainname."', '".$epp_REGISTRANT."', '".$epp_ADMIN."', '".$epp_TECH."', '".$epp_NAMESERVER."', '".$epp_AUTHINFO."', '', '".$epp_EXIPIRE."')";
				if (!mysql_query($query,$this->dblink)) {
					$this->dberror = "Query '".$query."' error: ".mysql_error();
					$this->dbclose();
					return $this->dberror;
				}
				$this->dbclose();
				return $this->svCode;
			} else {
				return $this->dberror;
			}
		} else {
			return $this->svCode;
		}
	} else {
		return $qrs;
	}
  }

  public function findREM($f,$type,$query,$found='tag') {
  global $basedir;
	$remstr = "";
	$result = @$f->xpath($query);
	while(list($k , $node) = each($result)) {
		if ($found == 'tag') {
			$remstr .= sprintf("%s\n", $result[$k]->asXML());
		} elseif ($found == 'node') {
			$remstr .= sprintf("%s\n", $node);
		}
	}
	return $remstr;
  }

  public function transferDomain($domainname) {
  global $basedir;
    $opmsg = "";
    $xml= file_get_contents($basedir.'infodomain.xml');
    $xml= str_replace('$$DOMAIN$$', $domainname, $xml);
	$this->xmlQuery= $xml;
	if ( $qrs= $this->ExecuteQuery("domain", "info") ) {
		if ($this->svCode == 1000) {
			$opmsg .= sprintf("\tGetting domain info:\t[OK]\n");
			$msgxml = $this->getMessageXML();
			$namespaces = $msgxml->getDocNamespaces(TRUE);
			foreach($namespaces as $a => $b) {
				$msgxml->registerXPathNamespace($a, $b);
			}
			$nsrem   = $this->findREM($msgxml,'ns','//domain:infData/domain:ns/domain:hostAttr');
			$techrem = $this->findREM($msgxml,'tech','//domain:infData/domain:contact[@type="tech"]');
			$updxml= file_get_contents($basedir.'changedns.xml');
			$updxml= str_replace('$$DOMAIN$$', $domainname, $updxml);
			$updxml= str_replace('$$DNSREM$$', $nsrem, $updxml);
			$updxml= str_replace('$$TECHREM$$', $techrem, $updxml);
			$opmsg .= sprintf("\tPrepare domain update command:\t[OK]\n");
			$this->xmlQuery= $updxml;
			if ( $updqrs= $this->ExecuteQuery("domain", "update") ) {
				if ($this->svCode != 1001) {
					$updmsgxml = $this->getMessageXML();
					$opmsg .= sprintf("\tDomain update command:\t[NOT OK]\n\t\tError: '%s'\n", $updmsgxml->asXML());
				} else {
					$opmsg .= sprintf("\tDomain update command:\t[OK]\n");
					return array($this->svCode, $opmsg);
				}
			} else {
				return array($updqrs, $opmsg);
			}
			return array($this->svCode, $opmsg);
		} else {
			$opmsg .= sprintf("\tGetting domain info:\t[NOT OK]\n");
			return array($this->svCode, $opmsg);
		}
	} else {
		return array($qrs, $opmsg);
	}
  }
  public function showCredit() {
    return $this->credit;
  }
}

main();

?>
